#directory "/home/yeting/kind/kind/src";;
#load "str.cma";;
#load "unix.cma";;
#load "counter.cmo";;
#load "types.cmo";;
#load "flags.cmo";;
#load "exceptions.cmo";;
#load "tables.cmo";;
#load "channels.cmo";;
#load "solver_base.cmo";;
#load "solver_yices.cmo";;
#load "solver_yices_cmdline.cmo";;
#load "solver_cvc3.cmo";;
#load "global.cmo";;
#load "deftable.cmo";;
#load "lus_convert_print.cmo";;
#load "lus_assertions.cmo";;
#load "lus_convert.cmo";;
#load "coi.cmo";;
#load "lus_convert_yc.cmo";;
#load "lus_types.cmo";;
#load "lustre_parser.cmo";;
#load "lustre_lexer.cmo";;
#load "abs.cmo";;
#load "lus_flatten.cmo";;
#load "inlining.cmo";;
#load "defgen.cmo";;
#load "kind_support.cmo";;
#load "kind_refine.cmo";;
#load "kind_compression.cmo";;
#load "util.cmo";;
#load "expr_util.cmo";;
#load "subexprs.cmo";;
#load "kind_ml.cmo";;
#load "kind.cmo";;
Kind.print_copyright();;



let pause_or_break hint break_echo = 
  print_string hint;
  let command = read_line () in
    if ("y" = command or "yes" = command) 
    then failwith break_echo
    else ()


open Types
open Exceptions
open Channels


let solver = Globals.my_solver

(* -------- from defgen.ml Yeting ----------------- *)
(* returns string buffer *)
(* ite_maxdepth should be > 0 for ite elim mode 'n' only *)
(* ite_maxdepth should be < 0 for ite elim mode 'i' only *)
let generate_var_defs_yc defhash def_base_name ite_maxdepth =
  let buf = Buffer.create 100 in
  Hashtbl.iter (fun x y ->
    match y with
        DEF_REF _ -> ()
      | DEF_STR dhe ->
          if ite_maxdepth >= 0 then
              solver#get#define_fun_buffer buf 
               (def_base_name^"__"^(string_of_int x)) (M_FUNC[M_NAT;M_BOOL])
               [(solver#get#position_var1,M_NAT)] dhe.def
          else
              solver#get#define_fun_buffer buf 
               (def_base_name^"__"^(string_of_int x)) M_BOOL [] dhe.def
    
  ) defhash;
  buf

(********************************************************************)
let gen_def filename =
  let use_file = open_in filename in
  let in_ch = use_file in
  let lexbuf = Lexing.from_channel in_ch in
  let outdoc = Buffer.create 1000 in
  let def_hash = Deftable.get_defhash() in
  let no_stateful = ref false in

  try
    let (cp,p,target_node) = Lustre_parser.main Lustre_lexer.token lexbuf in
(* cp is not currently used *)
    let position = POSITION_VAR (solver#get#position_var1) in

    if (!Flags.abstr_bool 
        || !Flags.abstr_ite 
        || !Flags.abstr_pre) then 
      Lus_flatten.flatten_all_node_defs ();


    let fd = Lus_convert.convert_def_list position target_node in
    let maxdepth = Lus_convert.get_max_depth() in
    if maxdepth > Lus_convert.get_max_adepth() then
      raise Unguarded_PRE;


    let fd' =
      if !Flags.aggressive_inlining > 0 then
        Inlining.inlined_formula fd (Coi.property_id_list p) 
      else
        fd
    in
    (* add constraints as assertions *)

    (* verfs lists the initial set of variables -- properties, basically *)
    Coi.examine_assignments fd';
    let vrefs1 = Coi.property_id_list p in
    let vrefs = 
      if !Flags.pre_refine_state_vars then
        List.rev_append vrefs1 (Lus_convert.state_vars_list())
      else
        vrefs1
    in
    Coi.calculate_all_dependencies vrefs 0 maxdepth;

      begin
debug_to_user "MAIN DEFGEN";
        let form_str_buf = 
          Lus_convert_yc.yc_formula_string_buffer GENERAL maxdepth fd' 
        in
        (* the above determin which variables are actually used *)
        let property = 
          Lus_convert_yc.yc_property_header solver#get#position_var1 position p 
        in
        let assertions =
          Lus_convert_yc.yc_assumption_string solver#get#position_var1 position
        in
        let vdef = Lus_convert_yc.yc_var_shortcut_string () in
        Buffer.add_string outdoc (solver#get#header_string^"\n");
        Buffer.add_string outdoc (solver#get#cc^"maxdepth = "^(string_of_int maxdepth)
          ^"\n");
        Buffer.add_string outdoc (vdef^"\n\n");

        Buffer.add_string outdoc (solver#get#cc^"Generic definitions:\n");
        (* print out generic def *)
        if (!Flags.var_defs_mode) then
          begin
            Deftable.initialize_defs def_hash vrefs; 
            Buffer.add_buffer outdoc (generate_var_defs_yc def_hash "DEF" 0);
          end
        else
          begin
            solver#get#aggdef_header outdoc form_str_buf;
          end;

        (* include ite-elim, coi, quant defs here *)
 
debug_to_user "PROPERTY DEFGEN";
        (* print out property & state limitations *)
(*        Buffer.add_string outdoc (property^"\n");*)
        Buffer.add_string outdoc (assertions^"\n");

        if (!Flags.loopfree_mode || !Flags.termination_check) then
          begin
            Coi.clean_used_vars maxdepth;
            try
              Buffer.add_string outdoc 
                ((Lus_convert_yc.yc_state_vars_string())^"\n");
            with NoStatefulVars ->
              no_stateful := true;
          end;

        Buffer.add_string outdoc (solver#get#cc^"BASE DONE\n")
      end; (* end yices *)
    (* return values *)
      (Buffer.contents outdoc),
      maxdepth,
      def_hash,
      !no_stateful,
      vrefs,
      target_node
  with TypeMismatch(s,x,y) ->
    Printf.printf "\nType Mismatch %s:\n%s\n!=\n%s\n at line %d (col %d)\n" s
      (solver#get#type_string x) (solver#get#type_string y) 
      (!Lus_convert.linenum) 
      ((Lexing.lexeme_start lexbuf) - !Lus_convert.linepos);
    raise Lus_convert_error
  | Parsing.Parse_error ->
    Printf.printf "\nParse error at line %d (col %d): '%s'\n" 
      (!Lus_convert.linenum) 
      ((Lexing.lexeme_start lexbuf) - !Lus_convert.linepos)
      (Lexing.lexeme lexbuf);
    raise Lus_convert_error
      

(* --- end ----- from defgen.ml Yeting ------- end ---------- *)


